@@ -162,7 +162,6 @@ bp_pipeline_bus_callback (GstBus *bus, GstMessage *message, gpointer userdata)
         
         case GST_MESSAGE_ELEMENT: {
             _bp_missing_elements_process_message (player, message);
-            _bp_vis_process_message (player, message);
             break;
         }
         
@@ -197,9 +196,12 @@ _bp_pipeline_construct (BansheePlayer *player)
     // fall back on auto, which should work on windows, and as a last ditch, try alsa
     audiosink = gst_element_factory_make ("gconfaudiosink", "audiosink");
     if (audiosink == NULL) {
-        audiosink = gst_element_factory_make ("autoaudiosink", "audiosink");
+        audiosink = gst_element_factory_make ("directsoundsink", "audiosink");
         if (audiosink == NULL) {
-            audiosink = gst_element_factory_make ("alsasink", "audiosink");
+            audiosink = gst_element_factory_make ("autoaudiosink", "audiosink");
+            if (audiosink == NULL) {
+                audiosink = gst_element_factory_make ("alsasink", "audiosink");
+            }
         }
     }
     
@@ -247,10 +249,6 @@ _bp_pipeline_construct (BansheePlayer *player)
     gst_element_add_pad (player->audiobin, gst_ghost_pad_new ("sink", teepad));
     gst_object_unref (teepad);
 
-    // Link the first tee pad to the primary audio sink queue
-    gst_pad_link (gst_element_get_request_pad (player->audiotee, "src0"), 
-        gst_element_get_pad (audiosinkqueue, "sink"));
-
     // Link the queue and the actual audio sink
     if (player->equalizer != NULL) {
         // link in equalizer, preamp and audioconvert.
@@ -261,7 +259,7 @@ _bp_pipeline_construct (BansheePlayer *player)
         gst_element_link (audiosinkqueue, audiosink);
     }
     
-    // _bp_vis_pipeline_setup (player);
+    _bp_vis_pipeline_setup (player);
     
     // Now that our internal audio sink is constructed, tell playbin to use it
     g_object_set (G_OBJECT (player->playbin), "audio-sink", player->audiobin, NULL);
@@ -274,6 +272,22 @@ _bp_pipeline_construct (BansheePlayer *player)
     _bp_cdda_pipeline_setup (player);
     _bp_video_pipeline_setup (player, bus);
 
+    // This call must be the last one in the pipeline setup to work around a
+    // GStreamer 0.10.21-0.10.22 algorithm that causes the last-allocated pad
+    // to be the one used for buffer allocations.  If the visualization one
+    // winds up being used for that then the pipeline will freeze when
+    // visualizations are disabled.
+    //
+    // When 0.10.23 is more mainstream we can use the new alloc-pad property to
+    // force selection of this pad for allocation.  Until then we just have to
+    // make sure it's the last one allocated.
+    //
+    // -- Chris Howie <cdhowie@gmail.com>
+
+    // Link the first tee pad to the primary audio sink queue
+    gst_pad_link (gst_element_get_request_pad (player->audiotee, "src0"),
+        gst_element_get_pad (audiosinkqueue, "sink"));
+
     return TRUE;
 }
 
