@@ -33,9 +33,16 @@ using Hyena.CommandLine;
 
 namespace Banshee.Base
 {
+    public delegate void InvokeHandler ();
+
     public static class ApplicationContext
     {
-        private static CommandLineParser command_line = new CommandLineParser ("enqueue");
+        static ApplicationContext () 
+        {
+            Log.Debugging = Debugging;
+        }
+    
+        private static CommandLineParser command_line = new CommandLineParser ();
         public static CommandLineParser CommandLine {
             set { command_line = value; }
             get { return command_line; }
@@ -52,6 +59,7 @@ namespace Banshee.Base
             get {
                 if (debugging == null) {
                     debugging = CommandLine.Contains ("debug");
+                    debugging |= CommandLine.Contains ("debug-sql");
                     debugging |= EnvironmentIsSet ("BANSHEE_DEBUG");
                 }
                 
@@ -64,13 +72,8 @@ namespace Banshee.Base
             return !String.IsNullOrEmpty (Environment.GetEnvironmentVariable (env));
         }
         
-        private static System.Globalization.CultureInfo culture_info = new System.Globalization.CultureInfo ("en-US");
         public static System.Globalization.CultureInfo InternalCultureInfo {
-            get { return culture_info; }
-        }
-
-        static ApplicationContext () {
-            Log.Debugging = Debugging;
+            get { return System.Globalization.CultureInfo.InvariantCulture; }
         }
     }
 }
