@@ -1,4 +1,3 @@
-
 // TrackInfo.cs
 //
 // Author:
@@ -45,7 +44,9 @@ namespace Banshee.Collection
     public class TrackInfo : CacheableItem, ITrackInfo
     {
         public const string ExportVersion = "1.0";
-    
+
+        public static readonly string UnknownTitle = Catalog.GetString ("Unknown Title");
+
         public class ExportableAttribute : Attribute
         {
             private string export_name;
@@ -57,7 +58,10 @@ namespace Banshee.Collection
     
         public delegate bool IsPlayingHandler (TrackInfo track);
         public static IsPlayingHandler IsPlayingMethod;
-            
+
+        public delegate void PlaybackFinishedHandler (TrackInfo track, double percentCompleted);
+        public static event PlaybackFinishedHandler PlaybackFinished;
+
         private SafeUri uri;
         private SafeUri more_info_uri;
         private string mimetype;
@@ -65,10 +69,14 @@ namespace Banshee.Collection
         private long file_mtime;
 
         private string artist_name;
+        private string artist_name_sort;
         private string album_title;
+        private string album_title_sort;
         private string album_artist;
+        private string album_artist_sort;
         private bool is_compilation;
         private string track_title;
+        private string track_title_sort;
         private string genre;
         private string composer;
         private string conductor;
@@ -84,6 +92,7 @@ namespace Banshee.Collection
         private int disc_count;
         private int year;
         private int rating;
+        private int score;
         private int bpm;
         private int bit_rate;
 
@@ -103,16 +112,27 @@ namespace Banshee.Collection
         {
         }
 
-        public virtual void IncrementPlayCount ()
+        public virtual void OnPlaybackFinished (double percentCompleted)
         {
-            LastPlayed = DateTime.Now;
-            PlayCount++;
-        }
+            double total_plays = PlayCount + SkipCount;
+            if (total_plays <= 0) {
+                Score = (int) Math.Floor ((Score + (percentCompleted * 100)) / 2);
+            } else {
+                Score = (int) Math.Floor ((((double)Score * total_plays) + (percentCompleted * 100)) / (total_plays + 1));
+            }
 
-        public virtual void IncrementSkipCount ()
-        {
-            LastSkipped = DateTime.Now;
-            SkipCount++;
+            if (percentCompleted <= 0.5) {
+                LastSkipped = DateTime.Now;
+                SkipCount++;
+            } else {
+                LastPlayed = DateTime.Now;
+                PlayCount++;
+            }
+
+            PlaybackFinishedHandler handler = PlaybackFinished;
+            if (handler != null) {
+                handler (this, percentCompleted);
+            }
         }
 
         public override string ToString ()
@@ -192,18 +212,36 @@ namespace Banshee.Collection
             set { artist_name = value; }
         }
 
+        [Exportable (ExportName = "artistsort")]
+        public virtual string ArtistNameSort {
+            get { return artist_name_sort; }
+            set { artist_name_sort = String.IsNullOrEmpty (value) ? null : value; }
+        }
+
         [Exportable (ExportName = "album")]
         public virtual string AlbumTitle {
             get { return album_title; }
             set { album_title = value; }
         }
 
+        [Exportable (ExportName = "albumsort")]
+        public virtual string AlbumTitleSort {
+            get { return album_title_sort; }
+            set { album_title_sort = String.IsNullOrEmpty (value) ? null : value; }
+        }
+
         [Exportable]
         public virtual string AlbumArtist {
-            get { return album_artist ?? (IsCompilation ? Catalog.GetString ("Various Artists") : ArtistName); }
+            get { return IsCompilation ? album_artist ?? Catalog.GetString ("Various Artists") : ArtistName; }
             set { album_artist = value; }
         }
         
+        [Exportable]
+        public virtual string AlbumArtistSort {
+            get { return album_artist_sort; }
+            set { album_artist_sort = String.IsNullOrEmpty (value) ? null : value; }
+        }
+        
         [Exportable]
         public virtual bool IsCompilation {
             get { return is_compilation; }
@@ -216,6 +254,12 @@ namespace Banshee.Collection
             set { track_title = value; }
         }
         
+        [Exportable (ExportName = "namesort")]
+        public virtual string TrackTitleSort {
+            get { return track_title_sort; }
+            set { track_title_sort = String.IsNullOrEmpty (value) ? null : value; }
+        }
+        
         [Exportable]
         public virtual string MusicBrainzId {
             get { return musicbrainz_id; }
@@ -237,43 +281,37 @@ namespace Banshee.Collection
             get { return release_date; }
             set { release_date = value; }
         }        
+
+        public virtual object ExternalObject {
+            get { return null; }
+        }
         
         public string DisplayArtistName { 
-            get {
-                string name = ArtistName == null ? null : ArtistName.Trim ();
-                return String.IsNullOrEmpty (name)
-                    ? Catalog.GetString ("Unknown Artist") 
-                    : name; 
-            } 
+            get { return StringUtil.MaybeFallback (ArtistName, ArtistInfo.UnknownArtistName); } 
+        }
+
+        public string DisplayAlbumArtistName {
+            get { return StringUtil.MaybeFallback (AlbumArtist, DisplayArtistName); }
         }
 
         public string DisplayAlbumTitle { 
-            get { 
-                string title = AlbumTitle == null ? null : AlbumTitle.Trim ();
-                return String.IsNullOrEmpty (title) 
-                    ? Catalog.GetString ("Unknown Album") 
-                    : title; 
-            } 
+            get { return StringUtil.MaybeFallback (AlbumTitle, AlbumInfo.UnknownAlbumTitle); } 
         }
 
         public string DisplayTrackTitle { 
-            get { 
-                string title = TrackTitle == null ? null : TrackTitle.Trim ();
-                return String.IsNullOrEmpty (title) 
-                    ? Catalog.GetString ("Unknown Title") 
-                    : title; 
-            } 
-        }     
+            get { return StringUtil.MaybeFallback (TrackTitle, UnknownTitle); } 
+        }
 
         public string DisplayGenre { 
             get { 
                 string genre = Genre == null ? null : Genre.Trim ();
-                return String.IsNullOrEmpty (genre) 
-                    ? Catalog.GetString ("Unknown Genre") 
-                    : genre; 
-            } 
-        }     
+                return String.IsNullOrEmpty (genre)
+                    ? String.Empty
+                    : genre;
+            }
+        }
         
+        [Exportable (ExportName = "artwork-id")]
         public virtual string ArtworkId { 
             get { return CoverArtSpec.CreateArtistAlbumId (AlbumArtist, AlbumTitle); }
         }
@@ -292,7 +330,7 @@ namespace Banshee.Collection
 
         [Exportable]
         public virtual int TrackCount {
-            get { return TrackNumber > track_count ? TrackNumber : track_count; }
+            get { return (track_count != 0 && track_count < TrackNumber) ? TrackNumber : track_count; }
             set { track_count = value; }
         }
 
@@ -304,7 +342,7 @@ namespace Banshee.Collection
         
         [Exportable]
         public virtual int DiscCount {
-            get { return DiscNumber > disc_count ? DiscNumber : disc_count; }
+            get { return (disc_count != 0 && disc_count < DiscNumber) ? DiscNumber : disc_count; }
             set { disc_count = value; }
         }
 
@@ -356,6 +394,12 @@ namespace Banshee.Collection
             set { rating = value; }
         }
         
+        [Exportable]
+        public virtual int Score {
+            get { return score; }
+            set {score = value; }
+        }
+
         [Exportable]
         public virtual int Bpm {
             get { return bpm; }
@@ -468,19 +512,42 @@ namespace Banshee.Collection
                 MediaAttributes |= attr;
             }
         }
-        
-        public string MediaTypeName {
+
+        // TODO turn this into a PrimarySource-owned delegate?
+        private static readonly string restart_podcast = Catalog.GetString ("_Restart Podcast");
+        private static readonly string restart_video = Catalog.GetString ("_Restart Video");
+        private static readonly string restart_song = Catalog.GetString ("_Restart Song");
+        private static readonly string restart_item = Catalog.GetString ("_Restart Item");
+
+        public string RestartLabel {
             get {
                 if (HasAttribute (TrackMediaAttributes.Podcast))
-                    return Catalog.GetString ("Podcast");
+                    return restart_podcast;
                 if (HasAttribute (TrackMediaAttributes.VideoStream))
-                    return Catalog.GetString ("Video");
+                    return restart_video;
                 if (HasAttribute (TrackMediaAttributes.Music))
-                    return Catalog.GetString ("Song");
-                return Catalog.GetString ("Item");
+                    return restart_song;
+                return restart_item;
             }
         }
-        
+
+        private static readonly string jump_to_podcast = Catalog.GetString ("_Jump to Playing Podcast");
+        private static readonly string jump_to_video = Catalog.GetString ("_Jump to Playing Video");
+        private static readonly string jump_to_song = Catalog.GetString ("_Jump to Playing Song");
+        private static readonly string jump_to_item = Catalog.GetString ("_Jump to Playing Item");
+
+        public string JumpToLabel {
+            get {
+                if (HasAttribute (TrackMediaAttributes.Podcast))
+                    return jump_to_podcast;
+                if (HasAttribute (TrackMediaAttributes.VideoStream))
+                    return jump_to_video;
+                if (HasAttribute (TrackMediaAttributes.Music))
+                    return jump_to_song;
+                return jump_to_item;
+            }
+        }
+
 #region Exportable Properties
         
         public static void ExportableMerge (TrackInfo source, TrackInfo dest)
