@@ -31,7 +31,6 @@ using System;
 using System.Data;
 using System.Text;
 using System.Collections.Generic;
-using System.Linq;
 
 using Mono.Unix;
 
@@ -43,28 +42,27 @@ using Hyena.Query;
 using Banshee.Base;
 using Banshee.Query;
 using Banshee.Database;
-using Banshee.PlaybackController;
 
 namespace Banshee.Collection.Database
-{
-    public class DatabaseTrackListModel : TrackListModel, IExportableModel,
-        ICacheableDatabaseModel, IFilterable, ISortable, ICareAboutView, ISearchable
+{       
+    public class DatabaseTrackListModel : TrackListModel, IExportableModel, 
+        ICacheableDatabaseModel, IFilterable, ISortable, ICareAboutView
     {
         private readonly BansheeDbConnection connection;
         private IDatabaseTrackModelProvider provider;
         protected IDatabaseTrackModelCache cache;
         private Banshee.Sources.DatabaseSource source;
-
+        
         private long count;
 
         private long filtered_count;
-        private TimeSpan filtered_duration, duration;
+        private TimeSpan filtered_duration;
         private long filtered_filesize, filesize;
-
+        
         private ISortableColumn sort_column;
         private string sort_query;
         private bool forced_sort_query;
-
+        
         private string reload_fragment;
         private string join_table, join_fragment, join_primary_key, join_column, condition, condition_from;
 
@@ -72,14 +70,14 @@ namespace Banshee.Collection.Database
         private string user_query;
 
         private int rows_in_view;
-
+        
         public DatabaseTrackListModel (BansheeDbConnection connection, IDatabaseTrackModelProvider provider, Banshee.Sources.DatabaseSource source)
         {
             this.connection = connection;
             this.provider = provider;
             this.source = source;
         }
-
+        
         protected HyenaSqliteConnection Connection {
             get { return connection; }
         }
@@ -95,7 +93,7 @@ namespace Banshee.Collection.Database
             cache.AggregatesUpdated += HandleCacheAggregatesUpdated;
             GenerateSortQueryPart ();
         }
-
+        
         private bool have_new_user_query = true;
         private void GenerateUserQueryFragment ()
         {
@@ -132,7 +130,7 @@ namespace Banshee.Collection.Database
         {
             SortQuery = (SortColumn == null || SortColumn.SortType == SortType.None)
                 ? (SortColumn != null && source is Banshee.Playlist.PlaylistSource)
-                    ? "CorePlaylistEntries.ViewOrder ASC, CorePlaylistEntries.EntryID ASC"
+                    ? "CorePlaylistEntries.ViewOrder ASC, CorePlaylistEntries.EntryID ASC" 
                     : BansheeQuery.GetSort ("Artist", true)
                 : BansheeQuery.GetSort (SortColumn.SortKey, SortColumn.SortType == SortType.Ascending);
         }
@@ -159,25 +157,17 @@ namespace Banshee.Collection.Database
             return true;
         }
 
-        public void Resort ()
-        {
-            var column = sort_column;
-            sort_column = null;
-            Sort (column);
-        }
-
         private void HandleCacheAggregatesUpdated (IDataReader reader)
         {
             filtered_duration = TimeSpan.FromMilliseconds (reader.IsDBNull (1) ? 0 : Convert.ToInt64 (reader[1]));
             filtered_filesize = reader.IsDBNull (2) ? 0 : Convert.ToInt64 (reader[2]);
         }
-
+        
         public override void Clear ()
         {
             cache.Clear ();
             count = 0;
             filesize = 0;
-            duration = TimeSpan.Zero;
             filtered_count = 0;
             OnCleared ();
         }
@@ -195,7 +185,7 @@ namespace Banshee.Collection.Database
         }
 
         private string unfiltered_query;
-        public virtual string UnfilteredQuery {
+        public string UnfilteredQuery {
             get {
                 return unfiltered_query ?? (unfiltered_query = String.Format (
                     "FROM {0} WHERE {1} {2}",
@@ -208,18 +198,7 @@ namespace Banshee.Collection.Database
 
         private string from;
         protected string From {
-            get {
-                if (from == null) {
-                    from = provider.From;
-                    int i = from.IndexOf (',');
-                    if (i > 0) {
-                        // Force the join order to fix bgo#581103 and bgo#603661
-                        // See section 5.2 in http://www.sqlite.org/optoverview.html
-                        from = from.Substring (0, i) + " CROSS JOIN " + from.Substring (i + 1);
-                    }
-                }
-                return from;
-            }
+            get { return from ?? provider.From; }
             set { from = value; }
         }
 
@@ -231,13 +210,12 @@ namespace Banshee.Collection.Database
         public virtual void UpdateUnfilteredAggregates ()
         {
             HyenaSqliteCommand count_command = new HyenaSqliteCommand (String.Format (
-                "SELECT COUNT(*), SUM(CoreTracks.FileSize), SUM(CoreTracks.Duration) {0}", UnfilteredQuery
+                "SELECT COUNT(*), SUM(CoreTracks.FileSize) {0}", UnfilteredQuery
             ));
-
+            
             using (HyenaDataReader reader = new HyenaDataReader (connection.Query (count_command))) {
                 count = reader.Get<long> (0);
                 filesize = reader.Get<long> (1);
-                duration = reader.Get<TimeSpan> (2);
             }
         }
 
@@ -278,12 +256,12 @@ namespace Banshee.Collection.Database
                     foreach (IFilterListModel model in reload_models) {
                         model.Reload (false);
                     }
-
+                    
                     bool have_filters = false;
                     foreach (IFilterListModel filter in source.CurrentFilters) {
                         have_filters |= !filter.Selection.AllSelected;
                     }
-
+                    
                     // Unless both artist/album selections are "all" (eg unfiltered), reload
                     // the track model again with the artist/album filters now in place.
                     if (have_filters) {
@@ -314,7 +292,7 @@ namespace Banshee.Collection.Database
         {
             StringBuilder qb = new StringBuilder ();
             qb.Append (UnfilteredQuery);
-
+            
             if (with_filters) {
                 foreach (IFilterListModel filter in source.CurrentFilters) {
                     string filter_sql = filter.GetSqlFilter ();
@@ -324,38 +302,21 @@ namespace Banshee.Collection.Database
                     }
                 }
             }
-
+            
             if (query_fragment != null) {
                 qb.Append (" AND ");
                 qb.Append (query_fragment);
             }
-
+            
             if (sort_query != null) {
                 qb.Append (" ORDER BY ");
                 qb.Append (sort_query);
             }
-
+            
             reload_fragment = qb.ToString ();
             cache.Reload ();
         }
 
-        private QueryFieldSet query_fields = BansheeQuery.FieldSet;
-        public QueryFieldSet QueryFields {
-            get { return query_fields; }
-            protected set { query_fields = value; }
-        }
-
-        public int IndexOf (QueryNode query, long offset)
-        {
-            lock (this) {
-                if (query == null) {
-                    return -1;
-                }
-
-                return (int) cache.IndexOf (query.ToSql (QueryFields), offset);
-            }
-        }
-
         public override int IndexOf (TrackInfo track)
         {
             lock (this) {
@@ -371,20 +332,31 @@ namespace Banshee.Collection.Database
         public int IndexOfFirst (TrackInfo track)
         {
             lock (this) {
-                return IndexOf (cache.GetSingleWhere ("AND MetadataHash = ? ORDER BY OrderID", track.MetadataHash));
+                return IndexOf (cache.GetSingle ("AND MetadataHash = ? ORDER BY OrderID", track.MetadataHash));
             }
         }
 
-        public override TrackInfo GetRandom (DateTime notPlayedSince)
-        {
-            return GetRandom (notPlayedSince, "song", true, false, Shuffler.Playback);
-        }
-
-        public TrackInfo GetRandom (DateTime notPlayedSince, string shuffle_mode, bool repeat, bool resetSinceTime, Shuffler shuffler)
+        private DateTime random_began_at = DateTime.MinValue;
+        private DateTime last_random = DateTime.MinValue;
+        private static string random_fragment = "AND (LastPlayedStamp < ? OR LastPlayedStamp IS NULL) AND (LastSkippedStamp < ? OR LastSkippedStamp IS NULL) ORDER BY RANDOM()";
+        public override TrackInfo GetRandom (DateTime notPlayedSince, bool repeat)
         {
             lock (this) {
-                shuffler.SetModelAndCache (this, cache);
-                return shuffler.GetRandom (notPlayedSince, shuffle_mode, repeat, resetSinceTime);
+                if (Count == 0)
+                    return null;
+
+                if (random_began_at < notPlayedSince)
+                    random_began_at = last_random = notPlayedSince;
+
+                TrackInfo track = cache.GetSingle (random_fragment, random_began_at, random_began_at);
+
+                if (track == null && repeat) {
+                    random_began_at = last_random;
+                    track = cache.GetSingle (random_fragment, random_began_at, random_began_at);
+                }
+
+                last_random = DateTime.Now;
+                return track;
             }
         }
 
@@ -400,22 +372,18 @@ namespace Banshee.Collection.Database
             get { return (int) filtered_count; }
         }
 
-        public virtual TimeSpan Duration {
+        public TimeSpan Duration {
             get { return filtered_duration; }
         }
 
-        public virtual long FileSize {
+        public long FileSize {
             get { return filtered_filesize; }
         }
-
+        
         public long UnfilteredFileSize {
             get { return filesize; }
         }
-
-        public TimeSpan UnfilteredDuration {
-            get { return duration; }
-        }
-
+        
         public int UnfilteredCount {
             get { return (int) count; }
             set { count = value; }
@@ -423,17 +391,17 @@ namespace Banshee.Collection.Database
 
         public string UserQuery {
             get { return user_query; }
-            set {
+            set { 
                 lock (this) {
-                    user_query = value;
+                    user_query = value; 
                     have_new_user_query = true;
                 }
             }
         }
-
+        
         public string ForcedSortQuery {
             get { return forced_sort_query ? sort_query : null; }
-            set {
+            set { 
                 forced_sort_query = value != null;
                 sort_query = value;
                 if (cache != null) {
@@ -468,7 +436,7 @@ namespace Banshee.Collection.Database
         {
             AddCondition (null, part);
         }
-
+        
         public void AddCondition (string tables, string part)
         {
             if (!String.IsNullOrEmpty (part)) {
@@ -479,7 +447,7 @@ namespace Banshee.Collection.Database
                 }
             }
         }
-
+        
         public string Condition {
             get { return condition; }
         }
@@ -519,20 +487,20 @@ namespace Banshee.Collection.Database
             get { return (int) cache.CacheId; }
         }
 
-        public ISortableColumn SortColumn {
+        public ISortableColumn SortColumn { 
             get { return sort_column; }
         }
-
+                
         public virtual int RowsInView {
             protected get { return rows_in_view; }
             set { rows_in_view = value; }
         }
 
-        int IExportableModel.GetLength ()
+        int IExportableModel.GetLength () 
         {
             return Count;
         }
-
+        
         IDictionary<string, object> IExportableModel.GetMetadata (int index)
         {
             return this[index].GenerateExportable ();
@@ -574,7 +542,7 @@ namespace Banshee.Collection.Database
         public string ReloadFragment {
             get { return reload_fragment; }
         }
-
+        
         public bool CachesValues { get { return false; } }
     }
 }
