@@ -34,19 +34,29 @@ using Mono.Unix;
 
 using Hyena;
 using Hyena.Data.Sqlite;
+
 using Banshee.Base;
 using Banshee.ServiceStack;
 using Banshee.Sources;
 using Banshee.Collection;
+using Banshee.Playlist;
 using Banshee.Collection.Database;
 using Banshee.Hardware;
 using Banshee.MediaEngine;
 using Banshee.MediaProfiles;
+using Banshee.Preferences;
+
+using Banshee.Dap.Gui;
 
 namespace Banshee.Dap
 {
     public abstract class DapSource : RemovableSource, IDisposable
     {
+        private DapSync sync;
+        private DapInfoBar dap_info_bar;
+        private Page page;
+        // private DapPropertiesDisplay dap_properties_display;
+        
         private IDevice device;
         internal IDevice Device {
             get { return device; }
@@ -58,6 +68,21 @@ namespace Banshee.Dap
             set { addin_id = value; }
         }
         
+        private MediaGroupSource music_group_source;
+        protected MediaGroupSource MusicGroupSource {
+            get { return music_group_source; }
+        }
+        
+        private MediaGroupSource video_group_source;
+        protected MediaGroupSource VideoGroupSource {
+            get { return video_group_source; }
+        }
+
+        private MediaGroupSource podcast_group_source;
+        protected MediaGroupSource PodcastGroupSource {
+            get { return podcast_group_source; }
+        }
+        
         protected DapSource ()
         {
         }
@@ -65,16 +90,30 @@ namespace Banshee.Dap
         public virtual void DeviceInitialize (IDevice device)
         {
             this.device = device;
-            type_unique_id = device.Uuid;
+            TypeUniqueId = device.Uuid;
         }
 
         public override void Dispose ()
         {
-            PurgeBuiltinSmartPlaylists ();
+            PurgeTemporaryPlaylists ();
             PurgeTracks ();
+            
+            if (dap_info_bar != null) {
+                dap_info_bar.Destroy ();
+                dap_info_bar = null;
+            }
+            
+            Properties.Remove ("Nereid.SourceContents.FooterWidget");
+            
+            /*Properties.Remove ("Nereid.SourceContents");
+            dap_properties_display.Destroy ();
+            dap_properties_display = null;*/
+
+            if (sync != null)
+                sync.Dispose ();
         }
         
-        private void PurgeBuiltinSmartPlaylists ()
+        private void PurgeTemporaryPlaylists ()
         {
             ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (@"
                 BEGIN TRANSACTION;
@@ -84,22 +123,63 @@ namespace Banshee.Dap
                 COMMIT TRANSACTION",
                 DbId, DbId
             ));
+
+            ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (@"
+                BEGIN TRANSACTION;
+                    DELETE FROM CorePlaylistEntries WHERE PlaylistID IN
+                        (SELECT PlaylistID FROM CorePlaylists WHERE PrimarySourceID = ?);
+                    DELETE FROM CorePlaylists WHERE PrimarySourceID = ?;   
+                COMMIT TRANSACTION",
+                DbId, DbId
+            ));
+        }
+        
+        internal void RaiseUpdated ()
+        {
+            OnUpdated ();
+        }
+
+        public virtual void SyncPlaylists ()
+        {
+        }
+
+        private bool supports_video = true;
+        public bool SupportsVideo {
+            get { return supports_video; }
+            protected set { supports_video = value; }
+        }
+
+        private bool supports_podcasts = true;
+        public bool SupportsPodcasts {
+            get { return supports_podcasts; }
+            protected set { supports_podcasts = value; }
         }
 
 #region Source
 
         protected override void Initialize ()
         {
-            PurgeBuiltinSmartPlaylists ();
+            PurgeTemporaryPlaylists ();
             
             base.Initialize ();
             
             Expanded = true;
             Properties.SetStringList ("Icon.Name", GetIconNames ());
             Properties.Set<string> ("SourcePropertiesActionLabel", Catalog.GetString ("Device Properties"));
-            Properties.Set<OpenPropertiesDelegate> ("SourceProperties.GuiHandler", delegate { 
-                new DapPropertiesDialog (this).RunDialog (); 
+            Properties.Set<OpenPropertiesDelegate> ("SourceProperties.GuiHandler", delegate {
+                new DapPropertiesDialog (this).RunDialog ();
             });
+            
+            Properties.Set<bool> ("Nereid.SourceContents.HeaderVisible", false);
+            Properties.Set<System.Reflection.Assembly> ("ActiveSourceUIResource.Assembly", System.Reflection.Assembly.GetExecutingAssembly ());
+            Properties.SetString ("ActiveSourceUIResource", "ActiveSourceUI.xml");
+
+            sync = new DapSync (this);
+            dap_info_bar = new DapInfoBar (this);
+            Properties.Set<Gtk.Widget> ("Nereid.SourceContents.FooterWidget", dap_info_bar);
+            
+            /*dap_properties_display = new DapPropertiesDisplay (this);
+            Properties.Set<Banshee.Sources.Gui.ISourceContents> ("Nereid.SourceContents", dap_properties_display);*/
 
             if (String.IsNullOrEmpty (GenericName)) {
                 GenericName = Catalog.GetString ("Media Player");
@@ -112,27 +192,52 @@ namespace Banshee.Dap
             AddDapProperty (Catalog.GetString ("Product"), device.Product);
             AddDapProperty (Catalog.GetString ("Vendor"), device.Vendor);
             
-            acceptable_mimetypes = MediaCapabilities != null 
-                ? MediaCapabilities.PlaybackMimeTypes 
-                : new string [] { "taglib/mp3" };
-        }
-        
-        public override void AddChildSource (Source child)
-        {
-            if (child is Banshee.Playlist.AbstractPlaylistSource && !(child is MediaGroupSource)) {
-                Log.Information ("Note: playlists added to digital audio players within Banshee are not yet saved to the device.", true);
+            if (acceptable_mimetypes == null) {
+                acceptable_mimetypes = HasMediaCapabilities 
+                    ? MediaCapabilities.PlaybackMimeTypes 
+                    : new string [] { "taglib/mp3" };
             }
             
-            base.AddChildSource (child);
+            AddChildSource (music_group_source = new MusicGroupSource (this));
+
+            if (SupportsVideo) {
+                video_group_source = new VideoGroupSource (this);
+            }
+
+            if (SupportsPodcasts) {
+                podcast_group_source = new PodcastGroupSource (this);
+            }
+
+            BuildPreferences ();
+
+            ThreadAssist.ProxyToMain (delegate {
+                Properties.Set<Banshee.Sources.Gui.ISourceContents> ("Nereid.SourceContents", new DapContent (this));
+            });
+        }
+
+        private void BuildPreferences ()
+        {
+            page = new Page ();
+            Section main_section = new Section ();
+            main_section.Order = -1;
+            
+            space_for_data = CreateSchema<long> ("space_for_data", 0, "How much space, in bytes, to reserve for data on the device.", "");
+            main_section.Add (space_for_data);
+            page.Add (main_section);
+
+            foreach (Section section in sync.PreferenceSections) {
+                page.Add (section);
+            }
+        }
+        
+        // Force to zero so that count doesn't show up
+        public override int Count {
+            get { return 0; }
         }
         
         public override bool HasProperties {
             get { return true; }
         }
-        
-        public override bool CanActivate {
-            get { return false; }
-        }
 
         public override void SetStatus (string message, bool can_close, bool is_spinning, string icon_name)
         {
@@ -153,34 +258,69 @@ namespace Banshee.Dap
 #endregion
         
 #region Track Management/Syncing   
-
-        internal void LoadDeviceContents ()
+ 
+        public void LoadDeviceContents ()
         {
             ThreadPool.QueueUserWorkItem (ThreadedLoadDeviceContents);
         }
         
         private void ThreadedLoadDeviceContents (object state)
         {
-            PurgeTracks ();
-            SetStatus (String.Format (Catalog.GetString ("Loading {0}"), Name), false);
-            LoadFromDevice ();
-            OnTracksAdded ();
-            HideStatus ();
-            
-            ThreadAssist.ProxyToMain (delegate {
-                DatabaseSource src;
-                AddChildSource (src = new MusicGroupSource (this));
-                src.Reload ();
-                AddChildSource (src = new VideoGroupSource (this));
-                src.Reload ();
-            });
+            try {
+                PurgeTracks ();
+                SetStatus (String.Format (Catalog.GetString ("Loading {0}"), Name), false);
+                LoadFromDevice ();
+                HideStatus ();
+
+                sync.DapLoaded ();
+                sync.CalculateSync ();
+                if (sync.AutoSync) {
+                    sync.Sync ();
+                }
+            } catch (Exception e) {
+                Log.Exception (e);
+            }
+        }
+
+        public void RemovePlaylists ()
+        {
+            // First remove any playlists on the device
+            List<Source> children = new List<Source> (sync.Dap.Children);
+            foreach (Source child in children) {
+                if (child is AbstractPlaylistSource && !(child is MediaGroupSource)) {
+                    (child as IUnmapableSource).Unmap ();
+                }
+            }
         }
 
         protected virtual void LoadFromDevice ()
         {
         }
+
+        protected override void Eject ()
+        {
+            if (!Sync.Enabled) {
+                // If sync isn't enabled, then make sure we've written saved our playlists
+                // Track transfers happen immediately, but playlists saves don't
+                SyncPlaylists ();
+            }
+        }
+
+        HyenaSqliteCommand track_on_dap_query = new HyenaSqliteCommand (
+            "SELECT TrackID FROM CoreTracks WHERE PrimarySourceID = ? AND MetadataHash = ? LIMIT 1");
+
+        private void AttemptToAddTrackToDevice (DatabaseTrackInfo track, SafeUri fromUri)
+        {
+            // Ensure there's enough space
+            if (Banshee.IO.File.Exists (fromUri) && BytesAvailable - Banshee.IO.File.GetSize (fromUri) >= 0) {
+                // Ensure it's not already on the device
+                if (ServiceManager.DbConnection.Query<int> (track_on_dap_query, DbId, track.MetadataHash) == 0) {
+                    AddTrackToDevice (track, fromUri);
+                }
+            }
+        }
         
-        protected abstract void AddTrackToDevice (DatabaseTrackInfo track, SafeUri fromUri);  
+        protected abstract void AddTrackToDevice (DatabaseTrackInfo track, SafeUri fromUri);
 
         protected bool TrackNeedsTranscoding (TrackInfo track)
         {
@@ -206,6 +346,11 @@ namespace Banshee.Dap
             dap_properties.Add (new DapProperty (key, val));
         }
 
+        protected void AddYesNoDapProperty (string key, bool val)
+        {
+            AddDapProperty (key, val ? Catalog.GetString ("Yes") : Catalog.GetString ("No"));
+        }
+
         public IEnumerable<DapProperty> DapProperties {
             get { return dap_properties; }
         }
@@ -213,7 +358,7 @@ namespace Banshee.Dap
         protected override void AddTrackAndIncrementCount (DatabaseTrackInfo track)
         {
             if (!TrackNeedsTranscoding (track)) {
-                AddTrackToDevice (track, track.Uri);
+                AttemptToAddTrackToDevice (track, track.Uri);
                 IncrementAddedTracks ();
                 return;
             }
@@ -224,16 +369,16 @@ namespace Banshee.Dap
                 string format = System.IO.Path.GetExtension (track.Uri.LocalPath);
                 format = String.IsNullOrEmpty (format) ? Catalog.GetString ("Unknown") : format.Substring (1);
                 throw new ApplicationException (String.Format (Catalog.GetString (
-                    "The {0} format is not supported by the device, and no converter was found to convert it."), format));
+                    "The {0} format is not supported by the device, and no converter was found to convert it"), format));
             }
 
             TranscoderService transcoder = ServiceManager.Get<TranscoderService> ();
             if (transcoder == null) {
                 throw new ApplicationException (Catalog.GetString (
-                    "File format conversion is not supported for this device."));
+                    "File format conversion support is not available"));
             }
             
-            transcoder.Enqueue (track, PreferredConfiguration, OnTrackTranscoded, OnTrackTranscodeCancelled);
+            transcoder.Enqueue (track, PreferredConfiguration, OnTrackTranscoded, OnTrackTranscodeCancelled, OnTrackTranscodeError);
         }
         
         private void OnTrackTranscoded (TrackInfo track, SafeUri outputUri)
@@ -241,7 +386,7 @@ namespace Banshee.Dap
             AddTrackJob.Status = String.Format ("{0} - {1}", track.ArtistName, track.TrackTitle);
             
             try {
-                AddTrackToDevice ((DatabaseTrackInfo)track, outputUri);
+                AttemptToAddTrackToDevice ((DatabaseTrackInfo)track, outputUri);
             } catch (Exception e) {
                 Log.Exception (e);
             }
@@ -253,6 +398,12 @@ namespace Banshee.Dap
         {
             IncrementAddedTracks (); 
         }
+
+        private void OnTrackTranscodeError (TrackInfo track)
+        {
+            ErrorSource.AddMessage (Catalog.GetString ("Error converting file"), track.Uri.ToString ());
+            IncrementAddedTracks ();
+        }
         
 #endregion
 
@@ -268,7 +419,7 @@ namespace Banshee.Dap
 
             if (!String.IsNullOrEmpty (vendor) && !String.IsNullOrEmpty (product)) {
                 return new string [] { 
-                    String.Format ("{0}-{1}", vendor, product).Replace (' ', '-').ToLower (), 
+                    String.Format ("multimedia-player-{0}-{1}", vendor, product).Replace (' ', '-').ToLower (), 
                     FallbackIcon
                 };
             } else {
@@ -276,7 +427,7 @@ namespace Banshee.Dap
             }
         }
         
-        private string FallbackIcon {
+        public static string FallbackIcon {
             get { return "multimedia-player"; }
         }
 
@@ -287,7 +438,15 @@ namespace Banshee.Dap
         protected virtual IDeviceMediaCapabilities MediaCapabilities {
             get { return device.MediaCapabilities; }
         }
-        
+
+        public Page Preferences {
+            get { return page; }
+        }
+
+        public DapSync Sync {
+            get { return sync; }
+        }
+
         private ProfileConfiguration preferred_config;
         private ProfileConfiguration PreferredConfiguration {
             get {
@@ -304,13 +463,56 @@ namespace Banshee.Dap
                 return preferred_config;
             }
         }
+        
+        internal protected virtual bool CanHandleDeviceCommand (DeviceCommand command)
+        {
+            return false;
+        }
 
         private string [] acceptable_mimetypes;
         public string [] AcceptableMimeTypes {
             get { return acceptable_mimetypes; }
             protected set { acceptable_mimetypes = value; }
         }
+       
+        public long BytesVideo {
+            get { return VideoGroupSource == null ? 0 : VideoGroupSource.BytesUsed; }
+        }
         
+        public long BytesMusic {
+            get { return MusicGroupSource == null ? 0 : MusicGroupSource.BytesUsed; }
+        }
+                    
+        public long BytesData {
+            get { return BytesUsed - BytesVideo - BytesMusic; }
+        }
+                    
+        public long BytesReserved {
+            get { return space_for_data.Get (); }
+            set { space_for_data.Set (value); }
+        }
+                    
+        public override long BytesAvailable {
+            get { return BytesCapacity - BytesUsed - Math.Max (0, BytesReserved - BytesData); }
+        }
+
+        public override bool CanRemoveTracks {
+            get { return base.CanRemoveTracks && !Sync.Enabled; }
+        }
+
+        public override bool CanDeleteTracks {
+            get { return base.CanDeleteTracks && !Sync.Enabled; }
+        }
+
+        public override bool CanAddTracks {
+            get { return base.CanAddTracks && !Sync.Enabled; }
+        }
+
+        public override bool PlaylistsReadOnly {
+            get { return Sync.Enabled || IsReadOnly; }
+        }
+            
+        private Banshee.Configuration.SchemaEntry<long> space_for_data;
 #endregion
         
     }
