@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Collections.Generic;
 using System.Threading;
 using Mono.Unix;
 
@@ -34,24 +35,29 @@ using Hyena;
 using Banshee.Base;
 using Banshee.ServiceStack;
 using Banshee.Sources;
+using Banshee.Library;
 using Banshee.Collection;
+using Banshee.Collection.Database;
 
 using Gtk;
 using Banshee.Gui;
 
 namespace Banshee.AudioCd
 {
-    public class AudioCdSource : Source, ITrackModelSource, IUnmapableSource, IDurationAggregator, IDisposable
+    public class AudioCdSource : Source, ITrackModelSource, IUnmapableSource, 
+        IImportSource, IDurationAggregator, IFileSizeAggregator, IDisposable
     {
         private AudioCdService service;
         private AudioCdDiscModel disc_model;
         private SourceMessage query_message;
-        
+
         public AudioCdSource (AudioCdService service, AudioCdDiscModel discModel) 
-            : base (Catalog.GetString ("Audio CD"), discModel.Title, 200)
+            : base (Catalog.GetString ("Audio CD"), discModel.Title, 400)
         {
             this.service = service;
             this.disc_model = discModel;
+
+            TypeUniqueId = "";
             
             Properties.SetString ("TrackView.ColumnControllerXml", String.Format (@"
                 <column-controller>
@@ -66,7 +72,7 @@ namespace Banshee.AudioCd
             disc_model.MetadataQueryFinished += OnMetadataQueryFinished;
             disc_model.EnabledCountChanged += OnEnabledCountChanged;
             disc_model.LoadModelFromDisc ();
-            
+
             SetupGui ();
         }
         
@@ -74,6 +80,10 @@ namespace Banshee.AudioCd
             get { return disc_model.Duration; }
         }
         
+        public long FileSize {
+            get { return disc_model.FileSize; }
+        }
+        
         public bool DiscIsPlaying {
             get {
                 AudioCdTrackInfo playing_track = ServiceManager.PlayerEngine.CurrentTrack as AudioCdTrackInfo;
@@ -81,10 +91,6 @@ namespace Banshee.AudioCd
             }
         }
         
-        public System.Collections.Generic.IEnumerable<Banshee.Collection.Database.IFilterListModel> FilterModels {
-            get { yield break; }
-        }
-        
         public void StopPlayingDisc ()
         {
             if (DiscIsPlaying) {
@@ -178,7 +184,7 @@ namespace Banshee.AudioCd
                     CoreTracks.AlbumID = CoreAlbums.AlbumID AND 
                     CoreArtists.Name = ? AND CoreAlbums.Title = ? AND (CoreTracks.Disc = ? OR CoreTracks.Disc = 0)",
                     ServiceManager.SourceManager.MusicLibrary.DbId,
-                    track.ArtistName, track.AlbumTitle, track.Disc
+                    track.ArtistName, track.AlbumTitle, track.DiscNumber
             );
 
             if (count > 0) {
@@ -219,7 +225,12 @@ namespace Banshee.AudioCd
 
         internal void DuplicateDisc ()
         {
-            Hyena.Log.Information ("This feature is not implemented yet.", true);
+            try {
+                AudioCdDuplicator.Duplicate (disc_model);
+            } catch (Exception e) {
+                Hyena.Log.Error (Catalog.GetString ("Could not duplicate audio CD"), e.Message, true);
+                Hyena.Log.Exception (e);
+            }
         }
         
         internal void LockAllTracks ()
@@ -230,7 +241,7 @@ namespace Banshee.AudioCd
                 track.CanPlay = false;
             }
             
-            OnUpdated ();
+            disc_model.NotifyUpdated ();
         }
         
         internal void UnlockAllTracks ()
@@ -239,24 +250,32 @@ namespace Banshee.AudioCd
                 track.CanPlay = true;
             }
             
-            OnUpdated ();
+            disc_model.NotifyUpdated ();
         }
         
         internal void UnlockTrack (AudioCdTrackInfo track)
         {
             track.CanPlay = true;
-            OnUpdated ();
+            disc_model.NotifyUpdated ();
         }
 
 #region Source Overrides
 
-        protected override string TypeUniqueId {
-            get { return "audio-cd"; }
-        }
-        
         public override int Count {
             get { return disc_model.Count; }
         }
+
+        public override string PreferencesPageId {
+            get { return "audio-cd"; }
+        }
+
+        public override bool HasEditableTrackProperties {
+            get { return true; }
+        }
+
+        public override bool HasViewableTrackProperties {
+            get { return true; }
+        }
         
 #endregion
         
@@ -302,7 +321,15 @@ namespace Banshee.AudioCd
         public bool ConfirmRemoveTracks {
             get { return false; }
         }
-        
+
+        public virtual bool CanRepeat {
+            get { return true; }
+        }
+
+        public virtual bool CanShuffle {
+            get { return true; }
+        }
+
         public bool ShowBrowser {
             get { return false; }
         }
@@ -311,6 +338,10 @@ namespace Banshee.AudioCd
             get { return false; }
         }
 
+        public bool Indexable {
+            get { return false; }
+        }
+
 #endregion
 
 #region IUnmapableSource Implementation
@@ -380,6 +411,7 @@ namespace Banshee.AudioCd
         private void SetupGui ()
         {                                       
             Properties.SetStringList ("Icon.Name", "media-cdrom", "gnome-dev-cdrom-audio", "source-cd-audio");
+            Properties.SetString ("SourcePreferencesActionLabel", Catalog.GetString ("Audio CD Preferences"));
             Properties.SetString ("UnmapSourceActionLabel", Catalog.GetString ("Eject Disc"));
             Properties.SetString ("UnmapSourceActionIconName", "media-eject");
             Properties.SetString ("ActiveSourceUIResource", "ActiveSourceUI.xml");
@@ -413,6 +445,7 @@ namespace Banshee.AudioCd
             Gtk.Action duplicate_action = uia_service.GlobalActions["DuplicateDiscAction"];
             if (duplicate_action != null) {
                 duplicate_action.IconName = "media-cdrom";
+                duplicate_action.Visible = AudioCdDuplicator.Supported;
             }
         }
         
@@ -425,6 +458,27 @@ namespace Banshee.AudioCd
             base.OnUpdated ();
         }
         
+#endregion
+
+#region IImportSource
+
+        void IImportSource.Import ()
+        {
+            ImportDisc ();
+        }
+        
+        string [] IImportSource.IconNames {
+            get { return Properties.GetStringList ("Icon.Name"); }
+        }
+        
+        bool IImportSource.CanImport {
+            get { return true; }
+        }
+        
+        int IImportSource.SortOrder {
+            get { return -10; }
+        }
+        
 #endregion
 
     }
