@@ -33,14 +33,14 @@ using Gtk;
 
 namespace Hyena.Data.Gui
 {
-    public partial class ListView<T> : Widget
+    public partial class ListView<T> : ListViewBase
     {
         public void SetModel (IListModel<T> model)
         {
             SetModel (model, 0.0);
         }
 
-        public void SetModel (IListModel<T> value, double vpos)
+        public virtual void SetModel (IListModel<T> value, double vpos)
         {
             if (model == value) {
                 return;
@@ -53,15 +53,23 @@ namespace Hyena.Data.Gui
             
             model = value;
 
-            ISortable sortable = model as ISortable;
-            if (sortable != null && sortable.SortColumn == null && ColumnController.DefaultSortColumn != null) {
-                sortable.Sort (ColumnController.DefaultSortColumn);
-            }
-
             if (model != null) {
                 model.Cleared += OnModelClearedHandler;
                 model.Reloaded += OnModelReloadedHandler;
                 selection_proxy.Selection = model.Selection;
+                IsEverReorderable = model.CanReorder;
+            }
+
+            ISortable sortable = model as ISortable;
+            if (sortable != null) {
+                ISortableColumn sort_column = ColumnController.SortColumn ?? ColumnController.DefaultSortColumn;
+                if (sort_column != null) {
+                    sortable.Sort (sort_column);
+                    RecalculateColumnSizes ();
+                    RegenerateColumnCache ();
+                    InvalidateHeader ();
+                    IsReorderable = sortable.SortColumn == null || sortable.SortColumn.SortType == SortType.None;
+                }
             }
             
             RefreshViewForModel (vpos);
@@ -69,37 +77,39 @@ namespace Hyena.Data.Gui
 
         private void RefreshViewForModel (double? vpos)
         {
+            if (Model == null) {
+                return;
+            }
+
             UpdateAdjustments ();
 
             if (vpos != null) {
                 ScrollTo ((double) vpos);
-            } else {
-                if (Model.Count <= RowsInView) {
-                    ScrollTo (0.0);
-                } else if (Selection.Count > 0) {
-                    bool selection_in_view = false;
-                    int first_row = GetRowAtY (0);
-                    for (int i = 0; i < RowsInView; i++) {
-                        if (Selection.Contains (first_row + i)) {
-                            selection_in_view = true;
-                            break;
-                        }
+            } else if (Model.Count <= RowsInView) {
+                // If our view fits all rows at once, make sure we're scrolled to the top
+                ScrollTo (0.0);
+            } else if (Selection != null && Selection.Count > 0 && !Selection.AllSelected) {
+                bool selection_in_view = false;
+                int first_row = GetRowAtY (0);
+                for (int i = 0; i < RowsInView; i++) {
+                    if (Selection.Contains (first_row + i)) {
+                        selection_in_view = true;
+                        break;
                     }
-
-                    if (!selection_in_view) {
-                        CenterOn (Selection.Ranges[0].Start);
-                    }
-                } else {
-                    ScrollTo (vadjustment.Value);
                 }
+
+                if (!selection_in_view) {
+                    CenterOn (Selection.Ranges[0].Start);
+                }
+            } else if (vadjustment != null) {
+                ScrollTo (vadjustment.Value);
             }
             
-            if (Model != null) {
+            if (Model != null && Selection != null) {
                 Selection.MaxIndex = Model.Count - 1;
             }
             
             if (Parent is ScrolledWindow) {
-                render_everything = true;
                 Parent.QueueDraw ();
             }
         }
@@ -171,6 +181,43 @@ namespace Hyena.Data.Gui
             return (bool)row_sensitive_property_info.GetValue (item, null);
         }
         
+        private string row_bold_property_name = "IsBold";
+        private PropertyInfo row_bold_property_info;
+        bool row_bold_property_invalid = false;
+        
+        public string RowBoldPropertyName {
+            get { return row_bold_property_name; }
+            set { 
+                if (value == row_bold_property_name) {
+                    return;
+                }
+                
+                row_bold_property_name = value;
+                row_bold_property_info = null;
+                row_bold_property_invalid = false;
+                
+                InvalidateList ();
+            }
+        }
+        
+        private bool IsRowBold (object item)
+        {
+            if (item == null || row_bold_property_invalid) {
+                return false;
+            }
+         
+            if (row_bold_property_info == null || row_bold_property_info.ReflectedType != item.GetType ()) {
+                row_bold_property_info = item.GetType ().GetProperty (row_bold_property_name);
+                if (row_bold_property_info == null || row_bold_property_info.PropertyType != typeof (bool)) {
+                    row_bold_property_info = null;
+                    row_bold_property_invalid = true;
+                    return false;
+                }
+            }
+            
+            return (bool)row_bold_property_info.GetValue (item, null);
+        }
+        
         #pragma warning disable 0169
         
         private bool IsRowSensitive (int index)
