@@ -53,14 +53,29 @@ namespace Hyena.Data.Gui
             get { return theme; }
         }
         
+        private bool changing_style = false;
+        
         protected override void OnStyleSet (Style old_style)
         {
+            if (changing_style) {
+                return;
+            }
+            
+            changing_style = true;
+            GtkUtilities.AdaptGtkRcStyle (this, typeof (TreeView));
+            changing_style = false;
+            
             base.OnStyleSet (old_style);
             RecomputeRowHeight = true;
-            theme = new GtkTheme (this);
+            theme = Hyena.Gui.Theming.ThemeEngine.CreateTheme (this);
 
             // Save the drawable so we can reuse it
             Gdk.Drawable drawable = cell_context != null ? cell_context.Drawable : null;
+            
+            if (pango_layout != null) {
+                pango_layout.Dispose ();
+                pango_layout = null;
+            }
 
             cell_context = new CellContext ();
             cell_context.Theme = theme;
@@ -76,7 +91,10 @@ namespace Hyena.Data.Gui
             }
             
             cairo_context = CairoHelper.Create (evnt.Window);
-            CairoExtensions.CreateLayout (this, cairo_context, ref pango_layout);
+            if (pango_layout == null) {
+                pango_layout = CairoExtensions.CreateLayout (this, cairo_context);
+            }
+            
             cell_context.Context = cairo_context;
             cell_context.Layout = pango_layout;
             
@@ -204,7 +222,7 @@ namespace Hyena.Data.Gui
             
             single_list_alloc.X = list_rendering_alloc.X - HadjustmentValue;
             single_list_alloc.Y = offset;
-            single_list_alloc.Width = list_rendering_alloc.Width;
+            single_list_alloc.Width = list_rendering_alloc.Width + HadjustmentValue;
             single_list_alloc.Height = RowHeight;
             
             int selection_height = 0;
@@ -321,7 +339,8 @@ namespace Hyena.Data.Gui
             
             if (pressed_column_is_dragging && pressed_column_index >= 0) {
                 cell_area.Width = column_cache[pressed_column_index].Width;
-                cell_area.X = pressed_column_x_drag + list_rendering_alloc.X - list_interaction_alloc.X;
+                cell_area.X = pressed_column_x_drag + list_rendering_alloc.X -
+                    list_interaction_alloc.X - HadjustmentValue;
                 PaintCell (item, pressed_column_index, row_index, cell_area, sensitive, bold, state, true);
             }
         }
